package TB2::Formatter::TAP;

use 5.008001;

use TB2::Mouse;
extends 'TB2::Formatter::TAP::v13';

our $VERSION = '1.005000_005';
$VERSION = eval $VERSION;    ## no critic (BuiltinFunctions::ProhibitStringyEval)


=head1 NAME

TB2::Formatter::TAP - Formatter as the latest stable TAP

=head1 SYNOPSIS

=head1 DESCRIPTION

This library is just a subclass of the latest stable version of TAP.
Currently TAP::v13.

=head1 SEE ALSO

L<< TB2::Formatter::TAP::v13 >>

=cut

1;
